/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package socialnetwork.ejb;

import java.util.List;
import javax.ejb.Stateless;
import javax.ejb.LocalBean;
import javax.persistence.EntityManager;
import javax.persistence.NoResultException;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;
import javax.persistence.TypedQuery;
import socialnetwork.entities.Blog;
import socialnetwork.entities.IGroup;
import socialnetwork.entities.ITextMedia;
import socialnetwork.entities.IUser;
import socialnetwork.entities.NetworkUser;
import socialnetwork.entities.Post;
import socialnetwork.entities.SubjectGroup;
import socialnetwork.entities.Topic;

/**
 * Governs operations with DB in application.
 * @author gsnewmark
 */
@Stateless
@LocalBean
public class SocNetworkDAO {
    @PersistenceContext(unitName = "UniversitySocialNetworkPU")
    private EntityManager em;
    
     /**
     * Checks whether supplied lecturer info exists in users table.
     * @param login
     * @param pass
     * @return 
     */
    public boolean checkAdminLogin(String login, String pass) {
        boolean passed = true;
        TypedQuery<NetworkUser> q = em.createQuery(
                "SELECT l FROM NetworkUser l WHERE l.isAdmin = true AND l.login = :name AND l.password = :pass",
                NetworkUser.class);
        q.setParameter("name", login);
        q.setParameter("pass", pass);
        try {
            q.getSingleResult();
        } catch (NoResultException e) {
            passed = false;
        }
        return passed;
    }
    
    /**
     * Checks whether supplied info exists in users table.
     * @param login
     * @param pass
     * @return null or logged user
     */
    public IUser checkUserLogin(String login, String pass) {
        IUser stud = null;
        TypedQuery<NetworkUser> q = em.createQuery(
                "SELECT l FROM NetworkUser l WHERE l.isAdmin = false AND l.login = :name AND l.password = :pass",
                NetworkUser.class);
        q.setParameter("name", login);
        q.setParameter("pass", pass);
        try {
            stud = q.getSingleResult();
        } catch (NoResultException e) {
            
        }
        return stud;
    }
    
    /**
     * Retrieves list of all subject groups from a DB.
     * @return list of all Subject groups
     */
    public List<SubjectGroup> getGroupsFromDB() {
        TypedQuery<SubjectGroup> q = em.createQuery(
                "SELECT g FROM SubjectGroup g", SubjectGroup.class);
        return q.getResultList();
    }
    
    /**
     * Retrieves subject group from DB.
     * @param id id of a group to retrieve
     * @return subject group
     */
    public SubjectGroup getGroupFromDB(long id) {
        return em.find(SubjectGroup.class, id);
    }
    
    /**
     * Retrieves user from DB.
     * @param id id of user in DB
     * @return Student object from DB
     */
    public IUser getUserFromDB(long id) {
        return em.find(NetworkUser.class, id);
    }
    
    /**
     * Retrieves blog from DB.
     * @param id id of blog in DB
     * @return Blog object from DB
     */
    public Blog getBlogFromDB(long id) {
        return em.find(Blog.class, id);
    }
    
    
    /**
     * Retrieves topic from DB.
     * @param id id of topic in DB
     * @return Topic object from DB
     */
    public Topic getTopicFromDB(long id) {
        return em.find(Topic.class, id);
    }
    
    /**
     * Retrieves post from DB.
     * @param id id of post in DB
     * @return Post object from DB
     */
    public Post getPostFromDB(int id) {
        return em.find(Post.class, id);
    }
    
    /**
     * @return quantity of students in DB
     */
    public long getStudentsQuantity() {
        Query q = em.createQuery(
                "SELECT COUNT(l) FROM NetworkUser l WHERE l.isAdmin = false");
        return (Long)q.getSingleResult();
    }
    
    /**
     * @return quantity of lecturers in DB
     */
    public long getLecturersQuantity() {
        Query q = em.createQuery(
                "SELECT COUNT(l) FROM NetworkUser l WHERE l.isAdmin = true");
        return (Long)q.getSingleResult();
    }
    
    /**
     * @return quantity of subject groups in DB
     */
    public long getGroupsQuantity() {
        Query q = em.createQuery(
                "SELECT COUNT(g) FROM SubjectGroup g");
        return (Long)q.getSingleResult();
    }
    
    /**
     * Gets list of all students.
     * @return list of students in DB
     */
    public List<NetworkUser> getStudentsList() {
        TypedQuery<NetworkUser> q = em.createQuery(
                "SELECT s FROM NetworkUser s WHERE s.isAdmin = false",
                NetworkUser.class);
        return q.getResultList();
    }
    
    /**
     * Adds user to DB.
     * @param user NetworkUser object
     * @param description description of a student
     */
    public void addUser(IUser user) {
        em.persist(user);
    }
    
    /**
     * Adds empty student to DB.
     */
    public void addEmptyStudent() {
        NetworkUser s = new NetworkUser();
        s.setIsAdmin(false);
        em.persist(s);
    }
    
    /**
     * Updates user's info in DB to newest state (saved in user).
     * @param user object with new state
     */
    public void updateUser(IUser user) {
        NetworkUser s = em.find(NetworkUser.class, user.getId());
        if (s != null) {
            em.merge(user);
        }
    }
    
    /**
     * Update existing user info or adds a new user.
     * @param user NetworkUser object to synchronize with DB
     */
    public void synchronizeUser(IUser user) {
        boolean inDB = false;
        if (user.getId() != null) {
            NetworkUser s = em.find(NetworkUser.class, user.getId());
            if (s != null) {
                inDB = true;
            } 
        }
        if (inDB) {
            em.merge(user);
        } else {
            em.persist(user);
        }
    }
    
    /**
     * Adds subject group to DB.
     * @param group SubjectGroup object to save in DB
     */
    public void addStudentGroup(IGroup group) {
        em.persist(group);
    }
    
    /**
     * Adds user to a subject group in DB.
     * @param user user to add
     * @param group group to add in
     */
    public void addUserToSubjectGroup(IUser user, IGroup group) {
        NetworkUser r = em.find(NetworkUser.class, user.getId());
        SubjectGroup g = em.find(SubjectGroup.class, group.getId());
        if (r != null && g != null) {
            user.getSubjectGroups().add((SubjectGroup)group);
            group.getUserList().add(user);
            em.merge(user);
            em.merge(group);
        }
    }
    
    /**
     * Removes user from a subject group in DB.
     * @param user user to remove
     * @param group group to remove from
     */
    public void removeUserFromSubjectGroup(IUser user, IGroup group) {
        NetworkUser r = em.find(NetworkUser.class, user.getId());
        SubjectGroup g = em.find(SubjectGroup.class, group.getId());
        if (r != null && g != null) {
            user.getSubjectGroups().remove((SubjectGroup)group);
            group.getUserList().remove(user);
            em.merge(user);
            em.merge(group);
        }
    }
    
    /**
     * Adds blog to DB.
     * @param blog Blog object to add to DB
     */
    public void addBlogToDB(ITextMedia blog) {
        em.persist(blog);
    }
    
    /**
     * Connects blog with specified group.
     * @param tMedia blog to add
     * @param group group to add to
     */
    public void connectGroupToBlog(ITextMedia tMedia, SubjectGroup group) {
        Blog b = em.find(Blog.class, tMedia.getId());
        SubjectGroup g = em.find(SubjectGroup.class, group.getId());
        if (b != null && g != null) {
            tMedia.setParentGroup(group);
            group.setBlog((Blog)tMedia);
            em.merge(tMedia);
            em.merge(group);
        }
    }
    
    /**
     * Disconnects blog from a group.
     * @param b_id id of a blog
     */
    public void disconnectGroupFromBlog(ITextMedia tMedia) {
        SubjectGroup group = tMedia.getParentGroup();
        Blog b = em.find(Blog.class, tMedia.getId());
        SubjectGroup g = em.find(SubjectGroup.class, group.getId());
        if (b != null && g != null) {
            group.setBlog(null);
            tMedia.setParentGroup(null);
            em.merge(tMedia);
            em.merge(group);
        }
    }
    
    /**
     * Adds topic to the DB.
     * @param topic Topic object to add
     */
    public void addTopicToDB(Topic topic) {
        em.persist(topic);
    }
    
    /**
     * Connects topic with a specified author.
     * @param topic topic to add
     * @param creator creator to connect with
     */
    public void addAuthorToTopic(Topic topic, IUser creator) {
        Topic t = em.find(Topic.class, topic.getId());
        NetworkUser u = em.find(NetworkUser.class, creator.getId());
        if (t != null && u != null) {
            topic.setCreator(creator);
            ((NetworkUser)creator).getTopics().add(topic);
            em.merge(topic);
            em.merge(creator);
        }
    }
    
    /**
     * Connects topic with a specified blog.
     * @param topic Topic object
     * @param tMedia Blog object
     */
    public void addBlogHolderToTopic(Topic topic, ITextMedia tMedia) {
        Blog b = em.find(Blog.class, tMedia.getId());
        Topic t = em.find(Topic.class, topic.getId());
        if (b != null && t != null) {
            tMedia.getTopics().add(topic);
            topic.setTopicHolder((Blog)tMedia);
            em.merge(tMedia);
            em.merge(topic);
        }
    }
    
    /**
     * Removes blog holder from a topic.
     * @param topic Topic object to remove blog from
     */
    public void removeBlogHolderFromTopic(Topic topic) {
        Blog blog = topic.getTopicHolder();
        Topic t = em.find(Topic.class, topic.getId());
        Blog b = em.find(Blog.class, blog.getId());
        if (b != null && t != null) {
            blog.getTopics().remove(topic);
            topic.setTopicHolder(null);
            em.merge(blog);
            em.merge(topic);
        }
    }
    
    /**
     * Adds post to the DB.
     * @param post Post object to add to DB
     */
    public void addPostToDB(Post post) {
        em.persist(post);
    }
    
    /**
     * Connects post with a specified topic.
     * @param post post to add
     * @param topic topic to add to
     */
    public void addTopicToPost(Post post, Topic topic) {
        Post p = em.find(Post.class, post.getId());
        Topic t = em.find(Topic.class, topic.getId());
        if (p != null && t != null) {
            post.setPostHolder(topic);
            topic.getPosts().add(post);
            em.merge(post);
            em.merge(topic);
        }
    }
    
    /**
     * Connects post with a specified author.
     * @param post post to add
     * @param creator creator to connect with
     */
    public void addAuthorToPost(Post post, IUser creator) {
        Post p = em.find(Post.class, post.getId());
        NetworkUser u = em.find(NetworkUser.class, creator.getId());
        if (p != null && u != null) {
            post.setCreator(creator);
            ((NetworkUser)creator).getPosts().add(post);
            em.merge(post);
            em.merge(creator);
        }
    }
    
     /**
     * Removes topic from a specified post.
     * @param post Post object to remove from it's topic
     */
    public void removeTopicFromPost(Post post) {
        Topic topic = post.getPostHolder();
        Topic t = em.find(Topic.class, topic.getId());
        Post p = em.find(Post.class, post.getId());
        if (p != null && t != null) {
            topic.getPosts().remove(post);
            post.setPostHolder(null);
            em.merge(post);
            em.merge(topic);
        }
    }
    
    /**
     * Deletes entry from blog table.
     * @param tMedia Blog object to delete
     */
    public void removeEntryFromBlog(ITextMedia tMedia) {
        Blog b = em.find(Blog.class, tMedia.getId());
        if (b != null) {
            em.remove(b);
        }
    }
    
    /**
     * Deletes entry from post table.
     * @param post Post object to delete
     */
    public void removeEntryFromPost(Post post) {
        Post p = em.find(Post.class, post.getId());
        if (p != null) {
            em.remove(p);
        }
    }
    
    /**
     * Deletes entry from subject group table.
     * @param group SubjectGroup object to delete
     */
    public void removeEntryFromSubjectGroup(IGroup group) {
        SubjectGroup g = em.find(SubjectGroup.class, group.getId());
        if (g != null) {
            em.remove(g);
        }
    }
    
    /**
     * Deletes entry from topic table.
     * @param topic Topic object to delete
     */
    public void removeEntryFromTopic(Topic topic) {
        Topic t = em.find(Topic.class, topic.getId());
        if (t != null) {
            em.remove(t);
        }
    }
    
    /**
     * Deletes entry from users table.
     * @param user NetworkUser object to delete
     */
    public void removeEntryFromUsers(IUser user) {
        NetworkUser u = em.find(NetworkUser.class, user.getId());
        if (u != null) {
            em.remove(u);
        }
    }
    
    public void createContents() {
        // Lectors
        NetworkUser l1 = new NetworkUser();
        l1.setIsAdmin(true);
        l1.setName("J. C. Denton");
        l1.setDescription("Implant specialist.");
        
        NetworkUser l2 = new NetworkUser();
        l2.setIsAdmin(true);
        l2.setName("R. von Belau");
        l2.setDescription("Herbalist, toledo.");
        
        // Students
        NetworkUser s1 = new NetworkUser();
        s1.setIsAdmin(false);
        s1.setName("G. Reau");
        s1.setDescription("FNS, 4, Lance velocity.");
        s1.setLogin("gr");
        s1.setPassword("gr");
        
        NetworkUser s2 = new NetworkUser();
        s2.setIsAdmin(false);
        s2.setName("B. Newmark");
        s2.setDescription("FI, 1, ICE-cracker.");
        s2.setLogin("bn");
        s2.setPassword("bn");
        
        NetworkUser s3 = new NetworkUser();
        s3.setIsAdmin(false);
        s3.setName("P. Dib");
        s3.setDescription("FPP, 3, Desert surviving.");
        s3.setLogin("pd");
        s3.setPassword("pd");
        
        NetworkUser s4 = new NetworkUser();
        s4.setIsAdmin(false);
        s4.setName("Kirk Roswell");
        s4.setDescription("FGK, 2, newman");
        s4.setLogin("kr");
        s4.setPassword("kr");
        
        // Groups
        SubjectGroup g1 = new SubjectGroup();
        g1.setName("Black computer magick");
        g1.getUserList().add(s2);
        s2.getSubjectGroups().add(g1);
        g1.getUserList().add(s3);
        s3.getSubjectGroups().add(g1);
        g1.getUserList().add(s1);
        s1.getSubjectGroups().add(g1);
        g1.getUserList().add(l2);
        l2.getSubjectGroups().add(g1);
        
        SubjectGroup g2 = new SubjectGroup();
        g2.setName("Testers Heaven");
        g2.getUserList().add(s2);
        s2.getSubjectGroups().add(g2);
        g2.getUserList().add(s4);
        s4.getSubjectGroups().add(g2);
        g2.getUserList().add(l1);
        l1.getSubjectGroups().add(g2);
        
        // Blogs
        Blog b1 = new Blog();
        b1.setMediaName("BCM News");
        b1.setCreator(l2);
        l1.getBlogs().add(b1);
        b1.setParentGroup(g1);
        g1.setBlog(b1);
        
        Blog b2 = new Blog();
        b2.setMediaName("Tricks&Treats");
        b2.setCreator(l1);
        l1.getBlogs().add(b2);
        b2.setParentGroup(g2);
        g2.setBlog(b2);
        
        // saving to db
        em.persist(l1);
        em.persist(l2);
        em.persist(s1);
        em.persist(s2);
        em.persist(s3);
        em.persist(s4);
        em.persist(g1);
        em.persist(g2);
        em.persist(b1);
        em.persist(b2);
    }
    
}
